@@ -39,6 +39,8 @@
 #include "gtksourceiter.h"
 #include "gtksourcestyleschememanager.h"
 #include "gtksourcestyle-private.h"
+#include "gtksourceundomanagerdefault.h"
+#include "gtksourceview-typebuiltins.h"
 
 /**
  * SECTION:buffer
@@ -88,6 +90,7 @@ enum {
 	SOURCE_MARK_UPDATED,
 	UNDO,
 	REDO,
+	BRACKET_MATCHED,
 	LAST_SIGNAL
 };
 
@@ -100,7 +103,8 @@ enum {
 	PROP_HIGHLIGHT_MATCHING_BRACKETS,
 	PROP_MAX_UNDO_LEVELS,
 	PROP_LANGUAGE,
-	PROP_STYLE_SCHEME
+	PROP_STYLE_SCHEME,
+	PROP_UNDO_MANAGER
 };
 
 struct _GtkSourceBufferPrivate
@@ -111,8 +115,9 @@ struct _GtkSourceBufferPrivate
 	gint                   constructed:1;
 
 	GtkTextTag            *bracket_match_tag;
-	GtkTextMark           *bracket_mark;
-	guint                  bracket_found:1;
+	GtkTextMark           *bracket_mark_cursor;
+	GtkTextMark           *bracket_mark_match;
+	GtkSourceBracketMatchType bracket_match;
 
 	GArray                *source_marks;
 
@@ -122,6 +127,9 @@ struct _GtkSourceBufferPrivate
 	GtkSourceStyleScheme  *style_scheme;
 
 	GtkSourceUndoManager  *undo_manager;
+	gint                   max_undo_levels;
+
+	gint                   allow_bracket_match:1;
 };
 
 G_DEFINE_TYPE (GtkSourceBuffer, gtk_source_buffer, GTK_TYPE_TEXT_BUFFER)
@@ -141,11 +149,9 @@ static void      gtk_source_buffer_get_property         (GObject
 							 guint                    prop_id,
 							 GValue                  *value,
 							 GParamSpec              *pspec);
-static void 	 gtk_source_buffer_can_undo_handler 	(GtkSourceUndoManager    *um,
-							 gboolean                 can_undo,
+static void 	 gtk_source_buffer_can_undo_handler 	(GtkSourceUndoManager    *manager,
 							 GtkSourceBuffer         *buffer);
-static void 	 gtk_source_buffer_can_redo_handler	(GtkSourceUndoManager    *um,
-							 gboolean                 can_redo,
+static void 	 gtk_source_buffer_can_redo_handler	(GtkSourceUndoManager    *manager,
 							 GtkSourceBuffer         *buffer);
 static void 	 gtk_source_buffer_real_insert_text 	(GtkTextBuffer           *buffer,
 							 GtkTextIter             *iter,
@@ -163,10 +169,17 @@ static void 	 gtk_source_buffer_real_delete_range 	(GtkTextBuffer           *buf
 static void 	 gtk_source_buffer_real_mark_set	(GtkTextBuffer		 *buffer,
 							 const GtkTextIter	 *location,
 							 GtkTextMark		 *mark);
+
+static void 	 gtk_source_buffer_real_apply_tag	(GtkTextBuffer		 *buffer,
+							 GtkTextTag		 *tag,
+							 const GtkTextIter	 *start,
+							 const GtkTextIter	 *end);
+
 static void 	 gtk_source_buffer_real_mark_deleted	(GtkTextBuffer		 *buffer,
 							 GtkTextMark		 *mark);
 static gboolean	 gtk_source_buffer_find_bracket_match_with_limit (GtkSourceBuffer *buffer,
 								  GtkTextIter     *orig,
+								  GtkSourceBracketMatchType *result,
 								  gint             max_chars);
 
 static void	 gtk_source_buffer_real_undo		(GtkSourceBuffer	 *buffer);
@@ -192,6 +205,7 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	tb_class->insert_text 	      = gtk_source_buffer_real_insert_text;
 	tb_class->insert_pixbuf       = gtk_source_buffer_real_insert_pixbuf;
 	tb_class->insert_child_anchor = gtk_source_buffer_real_insert_anchor;
+	tb_class->apply_tag           = gtk_source_buffer_real_apply_tag;
 
 	tb_class->mark_set	= gtk_source_buffer_real_mark_set;
 	tb_class->mark_deleted	= gtk_source_buffer_real_mark_deleted;
@@ -229,7 +243,8 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 	/**
 	 * GtkSourceBuffer:max-undo-levels:
 	 *
-	 * Number of undo levels for the buffer. -1 means no limit.
+	 * Number of undo levels for the buffer. -1 means no limit. This property
+	 * will only affect the default undo manager.
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_MAX_UNDO_LEVELS,
@@ -284,6 +299,14 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 							      GTK_TYPE_SOURCE_STYLE_SCHEME,
 							      G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+	                                 PROP_UNDO_MANAGER,
+	                                 g_param_spec_object ("undo-manager",
+	                                                      _("Undo manager"),
+	                                                      _("The buffer undo manager"),
+	                                                      GTK_TYPE_SOURCE_UNDO_MANAGER,
+	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+
 	param_types[0] = GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE;
 	param_types[1] = GTK_TYPE_TEXT_ITER | G_SIGNAL_TYPE_STATIC_SCOPE;
 
@@ -333,7 +356,72 @@ gtk_source_buffer_class_init (GtkSourceBufferClass *klass)
 			  G_TYPE_NONE,
 			  0);
 
-	g_type_class_add_private (object_class, sizeof(GtkSourceBufferPrivate));
+	/**
+	 * GtkSourceBuffer::bracket-matched
+	 * @buffer: a #GtkSourceBuffer.
+	 * @iter: iterator to initialize.
+	 * @state: state of bracket matching
+	 *
+	 * Sets @iter to a valid iterator pointing to the matching bracket
+	 * if @state is #GTK_SOURCE_BRACKET_MATCH_FOUND.
+	 *
+	 * Since: 2.12
+	 */
+	buffer_signals[BRACKET_MATCHED] =
+	    g_signal_new ("bracket-matched",
+			  G_OBJECT_CLASS_TYPE (object_class),
+			  G_SIGNAL_RUN_LAST,
+			  G_STRUCT_OFFSET (GtkSourceBufferClass, bracket_matched),
+			  NULL, NULL,
+			  _gtksourceview_marshal_VOID__BOXED_ENUM,
+			  G_TYPE_NONE, 2,
+			  GTK_TYPE_TEXT_ITER,
+			  GTK_TYPE_SOURCE_BRACKET_MATCH_TYPE);
+
+	g_type_class_add_private (object_class, sizeof (GtkSourceBufferPrivate));
+}
+
+static void
+set_undo_manager (GtkSourceBuffer      *buffer,
+                  GtkSourceUndoManager *manager)
+{
+	if (manager == buffer->priv->undo_manager)
+	{
+		return;
+	}
+
+	if (buffer->priv->undo_manager != NULL)
+	{
+		g_signal_handlers_disconnect_by_func (buffer->priv->undo_manager,
+		                                      G_CALLBACK (gtk_source_buffer_can_undo_handler),
+		                                      buffer);
+
+		g_signal_handlers_disconnect_by_func (buffer->priv->undo_manager,
+		                                      G_CALLBACK (gtk_source_buffer_can_redo_handler),
+		                                      buffer);
+
+		g_object_unref (buffer->priv->undo_manager);
+		buffer->priv->undo_manager = NULL;
+	}
+
+	if (manager != NULL)
+	{
+		buffer->priv->undo_manager = g_object_ref (manager);
+
+		g_signal_connect (buffer->priv->undo_manager,
+		                  "can-undo-changed",
+		                  G_CALLBACK (gtk_source_buffer_can_undo_handler),
+		                  buffer);
+
+		g_signal_connect (buffer->priv->undo_manager,
+		                  "can-redo-changed",
+		                  G_CALLBACK (gtk_source_buffer_can_redo_handler),
+		                  buffer);
+
+		/* Notify possible changes in the can-undo/redo state */
+		g_object_notify (G_OBJECT (buffer), "can-undo");
+		g_object_notify (G_OBJECT (buffer), "can-redo");
+	}
 }
 
 static void
@@ -346,27 +434,17 @@ gtk_source_buffer_init (GtkSourceBuffer *buffer)
 
 	buffer->priv = priv;
 
-	priv->undo_manager = gtk_source_undo_manager_new (GTK_TEXT_BUFFER (buffer));
-
 	priv->highlight_syntax = TRUE;
 	priv->highlight_brackets = TRUE;
-	priv->bracket_mark = NULL;
-	priv->bracket_found = FALSE;
+	priv->bracket_mark_cursor = NULL;
+	priv->bracket_mark_match = NULL;
+	priv->bracket_match = GTK_SOURCE_BRACKET_MATCH_NONE;
 
 	priv->source_marks = g_array_new (FALSE, FALSE, sizeof (GtkSourceMark *));
-
 	priv->style_scheme = _gtk_source_style_scheme_get_default ();
+
 	if (priv->style_scheme != NULL)
 		g_object_ref (priv->style_scheme);
-
-	g_signal_connect (priv->undo_manager,
-			  "can_undo",
-			  G_CALLBACK (gtk_source_buffer_can_undo_handler),
-			  buffer);
-	g_signal_connect (priv->undo_manager,
-			  "can_redo",
-			  G_CALLBACK (gtk_source_buffer_can_redo_handler),
-			  buffer);
 }
 
 static GObject *
@@ -375,13 +453,21 @@ gtk_source_buffer_constructor (GType                  type,
 			       GObjectConstructParam *construct_param)
 {
 	GObject *object;
+	GtkSourceBuffer *buffer;
 
 	object = G_OBJECT_CLASS(gtk_source_buffer_parent_class)->constructor (type,
 									      n_construct_properties,
 									      construct_param);
 
 	/* we need to know that the tag-table was set */
-	GTK_SOURCE_BUFFER (object)->priv->constructed = TRUE;
+	buffer = GTK_SOURCE_BUFFER (object);
+	buffer->priv->constructed = TRUE;
+
+	if (buffer->priv->undo_manager == NULL)
+	{
+		/* This will install the default undo manager */
+		gtk_source_buffer_set_undo_manager (buffer, NULL);
+	}
 
 	return object;
 }
@@ -416,8 +502,7 @@ gtk_source_buffer_dispose (GObject *object)
 
 	if (buffer->priv->undo_manager != NULL)
 	{
-		g_object_unref (buffer->priv->undo_manager);
-		buffer->priv->undo_manager = NULL;
+		set_undo_manager (buffer, NULL);
 	}
 
 	if (buffer->priv->highlight_engine != NULL)
@@ -481,6 +566,11 @@ gtk_source_buffer_set_property (GObject      *object,
 							    g_value_get_object (value));
 			break;
 
+		case PROP_UNDO_MANAGER:
+			gtk_source_buffer_set_undo_manager (source_buffer,
+			                                    g_value_get_object (value));
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -513,7 +603,7 @@ gtk_source_buffer_get_property (GObject    *object,
 
 		case PROP_MAX_UNDO_LEVELS:
 			g_value_set_int (value,
-					 gtk_source_buffer_get_max_undo_levels (source_buffer));
+					 source_buffer->priv->max_undo_levels);
 			break;
 
 		case PROP_LANGUAGE:
@@ -532,6 +622,10 @@ gtk_source_buffer_get_property (GObject    *object,
 			g_value_set_boolean (value, gtk_source_buffer_can_redo (source_buffer));
 			break;
 
+		case PROP_UNDO_MANAGER:
+			g_value_set_object (value, source_buffer->priv->undo_manager);
+			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -540,7 +634,7 @@ gtk_source_buffer_get_property (GObject    *object,
 
 /**
  * gtk_source_buffer_new:
- * @table: a #GtkTextTagTable, or %NULL to create a new one.
+ * @table: (allow-none): a #GtkTextTagTable, or %NULL to create a new one.
  *
  * Creates a new source buffer.
  *
@@ -568,21 +662,17 @@ gtk_source_buffer_new (GtkTextTagTable *table)
 GtkSourceBuffer *
 gtk_source_buffer_new_with_language (GtkSourceLanguage *language)
 {
-	GtkSourceBuffer *buffer;
-
 	g_return_val_if_fail (GTK_IS_SOURCE_LANGUAGE (language), NULL);
 
-	buffer = gtk_source_buffer_new (NULL);
-
-	gtk_source_buffer_set_language (buffer, language);
-
-	return buffer;
+	return g_object_new (GTK_TYPE_SOURCE_BUFFER,
+			     "tag-table", NULL,
+			     "language", language,
+			     NULL);
 }
 
 static void
-gtk_source_buffer_can_undo_handler (GtkSourceUndoManager  	*um,
-				    gboolean			 can_undo,
-				    GtkSourceBuffer 		*buffer)
+gtk_source_buffer_can_undo_handler (GtkSourceUndoManager *manager,
+                                    GtkSourceBuffer      *buffer)
 {
 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
 
@@ -590,9 +680,8 @@ gtk_source_buffer_can_undo_handler (GtkSourceUndoManager  	*um,
 }
 
 static void
-gtk_source_buffer_can_redo_handler (GtkSourceUndoManager  	*um,
-				    gboolean         		 can_redo,
-				    GtkSourceBuffer 		*buffer)
+gtk_source_buffer_can_redo_handler (GtkSourceUndoManager *manager,
+                                    GtkSourceBuffer      *buffer)
 {
 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
 
@@ -639,12 +728,67 @@ _gtk_source_buffer_get_bracket_match_tag (GtkSourceBuffer *buffer)
 	return buffer->priv->bracket_match_tag;
 }
 
+static gunichar
+bracket_pair (gunichar base_char, gint *direction)
+{
+	gint dir;
+	gunichar pair;
+
+	switch ((int)base_char)
+	{
+		case '{':
+			dir = 1;
+			pair = '}';
+			break;
+		case '(':
+			dir = 1;
+			pair = ')';
+			break;
+		case '[':
+			dir = 1;
+			pair = ']';
+			break;
+		case '<':
+			dir = 1;
+			pair = '>';
+			break;
+		case '}':
+			dir = -1;
+			pair = '{';
+			break;
+		case ')':
+			dir = -1;
+			pair = '(';
+			break;
+		case ']':
+			dir = -1;
+			pair = '[';
+			break;
+		case '>':
+			dir = -1;
+			pair = '<';
+			break;
+		default:
+			dir = 0;
+			pair = 0;
+			break;
+	}
+
+	/* Let direction be NULL if we don't care */
+	if (direction != NULL)
+		*direction = dir;
+
+	return pair;
+}
+
 static void
 gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 			       const GtkTextIter *iter,
 			       GtkTextMark       *mark)
 {
-	GtkTextIter iter1, iter2;
+	GtkSourceBuffer *source_buffer;
+	GtkTextIter start, end;
+	gunichar cursor_char;
 
 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
 	g_return_if_fail (iter != NULL);
@@ -654,49 +798,106 @@ gtk_source_buffer_move_cursor (GtkTextBuffer     *buffer,
 	if (mark != gtk_text_buffer_get_insert (buffer))
 		return;
 
-	if (GTK_SOURCE_BUFFER (buffer)->priv->bracket_found)
+	source_buffer = GTK_SOURCE_BUFFER (buffer);
+
+	if (source_buffer->priv->bracket_match == GTK_SOURCE_BRACKET_MATCH_FOUND)
 	{
 		gtk_text_buffer_get_iter_at_mark (buffer,
-						  &iter1,
-						  GTK_SOURCE_BUFFER (buffer)->priv->bracket_mark);
-		iter2 = iter1;
-		gtk_text_iter_forward_char (&iter2);
+						  &start,
+						  source_buffer->priv->bracket_mark_match);
+
+		gtk_text_buffer_get_iter_at_mark (buffer,
+						  &end,
+						  source_buffer->priv->bracket_mark_cursor);
+
+		gtk_text_iter_order (&start, &end);
+		gtk_text_iter_forward_char (&end);
 		gtk_text_buffer_remove_tag (buffer,
-					    GTK_SOURCE_BUFFER (buffer)->priv->bracket_match_tag,
-					    &iter1,
-					    &iter2);
+					    source_buffer->priv->bracket_match_tag,
+					    &start,
+					    &end);
 	}
 
-	if (!GTK_SOURCE_BUFFER (buffer)->priv->highlight_brackets)
+	if (!source_buffer->priv->highlight_brackets)
 		return;
 
-	iter1 = *iter;
-	if (gtk_source_buffer_find_bracket_match_with_limit (GTK_SOURCE_BUFFER (buffer),
-	                                                     &iter1,
-	                                                     MAX_CHARS_BEFORE_FINDING_A_MATCH))
+	start = *iter;
+	if (!gtk_source_buffer_find_bracket_match_with_limit (source_buffer,
+	                                                      &start,
+	                                                      &source_buffer->priv->bracket_match,
+	                                                      MAX_CHARS_BEFORE_FINDING_A_MATCH))
 	{
-		if (!GTK_SOURCE_BUFFER (buffer)->priv->bracket_mark)
-			GTK_SOURCE_BUFFER (buffer)->priv->bracket_mark =
-				gtk_text_buffer_create_mark (buffer,
-							     NULL,
-							     &iter1,
-							     FALSE);
-		else
-			gtk_text_buffer_move_mark (buffer,
-						   GTK_SOURCE_BUFFER (buffer)->priv->bracket_mark,
-						   &iter1);
-
-		iter2 = iter1;
-		gtk_text_iter_forward_char (&iter2);
-		gtk_text_buffer_apply_tag (buffer,
-					   get_bracket_match_tag (GTK_SOURCE_BUFFER (buffer)),
-					   &iter1,
-					   &iter2);
-		GTK_SOURCE_BUFFER (buffer)->priv->bracket_found = TRUE;
+		g_signal_emit (source_buffer,
+			       buffer_signals[BRACKET_MATCHED],
+			       0,
+			       NULL,
+			       &source_buffer->priv->bracket_match);
 	}
 	else
 	{
-		GTK_SOURCE_BUFFER (buffer)->priv->bracket_found = FALSE;
+		g_signal_emit (source_buffer,
+			       buffer_signals[BRACKET_MATCHED],
+			       0,
+			       &start,
+			       &source_buffer->priv->bracket_match);
+
+		/* allow_bracket_match will allow the bracket match tag to be
+		   applied to the buffer. See apply_tag_real for more
+		   information */
+		source_buffer->priv->allow_bracket_match = TRUE;
+		
+		/* Mark matching bracket */
+		if (!source_buffer->priv->bracket_mark_match)
+		{
+			source_buffer->priv->bracket_mark_match =
+ 				gtk_text_buffer_create_mark (buffer,
+							     NULL,
+							     &start,
+							     TRUE);
+ 		}
+ 		else
+ 		{
+ 			gtk_text_buffer_move_mark (buffer,
+						   source_buffer->priv->bracket_mark_match,
+						   &start);
+ 		}
+
+		end = start;
+		gtk_text_iter_forward_char (&end);
+		gtk_text_buffer_apply_tag (buffer,
+					   get_bracket_match_tag (source_buffer),
+					   &start,
+					   &end);
+
+		/* Mark the bracket near the cursor */
+		start = *iter;
+		cursor_char = gtk_text_iter_get_char (&start);
+		if (bracket_pair (cursor_char, NULL) == 0)
+			gtk_text_iter_backward_char (&start);
+
+		if (!source_buffer->priv->bracket_mark_cursor)
+		{
+			source_buffer->priv->bracket_mark_cursor =
+				gtk_text_buffer_create_mark (buffer,
+							     NULL,
+							     &start,
+							     FALSE);
+		}
+		else
+		{
+			gtk_text_buffer_move_mark (buffer,
+						   source_buffer->priv->bracket_mark_cursor,
+						   &start);
+		}
+
+		end = start;
+		gtk_text_iter_forward_char (&end);
+		gtk_text_buffer_apply_tag (buffer,
+					   get_bracket_match_tag (source_buffer),
+					   &start,
+					   &end);
+
+		source_buffer->priv->allow_bracket_match = FALSE;
 	}
 }
 
@@ -861,9 +1062,10 @@ get_context_class_mask (GtkSourceBuffer *buffer,
 }
 
 static gboolean
-gtk_source_buffer_find_bracket_match_real (GtkSourceBuffer *buffer,
-                                           GtkTextIter     *orig,
-                                           gint             max_chars)
+gtk_source_buffer_find_bracket_match_real (GtkSourceBuffer           *buffer,
+                                           GtkTextIter               *orig,
+					   GtkSourceBracketMatchType *result,
+                                           gint                       max_chars)
 {
 	GtkTextIter iter;
 
@@ -885,52 +1087,20 @@ gtk_source_buffer_find_bracket_match_real (GtkSourceBuffer *buffer,
 	base_char = search_char = cur_char;
 	cclass_mask = get_context_class_mask (buffer, &iter);
 
-	switch ((int) base_char) {
-		case '{':
-			addition = 1;
-			search_char = '}';
-			break;
-		case '(':
-			addition = 1;
-			search_char = ')';
-			break;
-		case '[':
-			addition = 1;
-			search_char = ']';
-			break;
-		case '<':
-			addition = 1;
-			search_char = '>';
-			break;
-		case '}':
-			addition = -1;
-			search_char = '{';
-			break;
-		case ')':
-			addition = -1;
-			search_char = '(';
-			break;
-		case ']':
-			addition = -1;
-			search_char = '[';
-			break;
-		case '>':
-			addition = -1;
-			search_char = '<';
-			break;
-		default:
-			addition = 0;
-			break;
-	}
+	search_char = bracket_pair (base_char, &addition);
 
 	if (addition == 0)
+	{
+		*result = GTK_SOURCE_BRACKET_MATCH_NONE;
 		return FALSE;
+	}
 
 	counter = 0;
 	found = FALSE;
 	char_cont = 0;
 
-	do {
+	do
+	{
 		gint current_mask;
 
 		gtk_text_iter_forward_chars (&iter, addition);
@@ -950,7 +1120,8 @@ gtk_source_buffer_find_bracket_match_real (GtkSourceBuffer *buffer,
 		if ((cur_char == search_char || cur_char == base_char) &&
 		    cclass_mask == current_mask)
 		{
-			if ((cur_char == search_char) && counter == 0) {
+			if ((cur_char == search_char) && counter == 0)
+			{
 				found = TRUE;
 				break;
 			}
@@ -964,7 +1135,18 @@ gtk_source_buffer_find_bracket_match_real (GtkSourceBuffer *buffer,
 		((char_cont < max_chars) || (max_chars < 0)));
 
 	if (found)
+	{
 		*orig = iter;
+		*result = GTK_SOURCE_BRACKET_MATCH_FOUND;
+	}
+	else if (char_cont >= max_chars && max_chars >= 0)
+	{
+		*result = GTK_SOURCE_BRACKET_MATCH_OUT_OF_RANGE;
+	}
+	else
+	{
+		*result = GTK_SOURCE_BRACKET_MATCH_NOT_FOUND;
+	}
 
 	return found;
 }
@@ -974,13 +1156,14 @@ gtk_source_buffer_find_bracket_match_real (GtkSourceBuffer *buffer,
  * cursor takes precedence.
  */
 static gboolean
-gtk_source_buffer_find_bracket_match_with_limit (GtkSourceBuffer *buffer,
-                                                 GtkTextIter     *orig,
-                                                 gint             max_chars)
+gtk_source_buffer_find_bracket_match_with_limit (GtkSourceBuffer           *buffer,
+                                                 GtkTextIter               *orig,
+                                                 GtkSourceBracketMatchType *result,
+                                                 gint                       max_chars)
 {
 	GtkTextIter iter;
 
-	if (gtk_source_buffer_find_bracket_match_real (buffer, orig, max_chars))
+	if (gtk_source_buffer_find_bracket_match_real (buffer, orig, result, max_chars))
 	{
 		return TRUE;
 	}
@@ -989,7 +1172,7 @@ gtk_source_buffer_find_bracket_match_with_limit (GtkSourceBuffer *buffer,
 	if (!gtk_text_iter_starts_line (&iter) &&
 	    gtk_text_iter_backward_char (&iter))
 	{
-		if (gtk_source_buffer_find_bracket_match_real (buffer, &iter, max_chars))
+		if (gtk_source_buffer_find_bracket_match_real (buffer, &iter, result, max_chars))
 		{
 			*orig = iter;
 			return TRUE;
@@ -999,29 +1182,6 @@ gtk_source_buffer_find_bracket_match_with_limit (GtkSourceBuffer *buffer,
 	return FALSE;
 }
 
-#if 0
-/**
- * gtk_source_iter_find_matching_bracket:
- * @iter: a #GtkTextIter.
- *
- * Tries to match the bracket character currently at @iter with its
- * opening/closing counterpart, and if found moves @iter to the position
- * where it was found.
- *
- * @iter must be a #GtkTextIter belonging to a #GtkSourceBuffer.
- *
- * Return value: %TRUE if the matching bracket was found and the @iter
- * iter moved.
- **/
-gboolean
-gtk_source_iter_find_matching_bracket (GtkTextIter *iter)
-{
-	g_return_val_if_fail (iter != NULL, FALSE);
-
-	return gtk_source_buffer_find_bracket_match_with_limit (iter, -1);
-}
-#endif
-
 /**
  * gtk_source_buffer_can_undo:
  * @buffer: a #GtkSourceBuffer.
@@ -1106,7 +1266,7 @@ gtk_source_buffer_get_max_undo_levels (GtkSourceBuffer *buffer)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), 0);
 
-	return gtk_source_undo_manager_get_max_undo_levels (buffer->priv->undo_manager);
+	return buffer->priv->max_undo_levels;
 }
 
 /**
@@ -1133,13 +1293,20 @@ gtk_source_buffer_set_max_undo_levels (GtkSourceBuffer *buffer,
 {
 	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
 
-	if (gtk_source_undo_manager_get_max_undo_levels (
-		    buffer->priv->undo_manager) != max_undo_levels)
+	if (buffer->priv->max_undo_levels == max_undo_levels)
 	{
-		gtk_source_undo_manager_set_max_undo_levels (buffer->priv->undo_manager,
-							     max_undo_levels);
-		g_object_notify (G_OBJECT (buffer), "max-undo-levels");
+		return;
 	}
+
+	buffer->priv->max_undo_levels = max_undo_levels;
+
+	if (GTK_IS_SOURCE_UNDO_MANAGER_DEFAULT (buffer->priv->undo_manager))
+	{
+		gtk_source_undo_manager_default_set_max_undo_levels (GTK_SOURCE_UNDO_MANAGER_DEFAULT (buffer->priv->undo_manager),
+		                                                     max_undo_levels);
+	}
+
+	g_object_notify (G_OBJECT (buffer), "max-undo-levels");
 }
 
 /**
@@ -1286,9 +1453,9 @@ gtk_source_buffer_set_highlight_syntax (GtkSourceBuffer *buffer,
 /**
  * gtk_source_buffer_set_language:
  * @buffer: a #GtkSourceBuffer.
- * @language: a #GtkSourceLanguage to set, or %NULL.
+ * @language: (allow-none): a #GtkSourceLanguage to set, or %NULL.
  *
- * Associate a #GtkSourceLanguage with the source buffer. If @language is
+ * Associate a #GtkSourceLanguage with the buffer. If @language is
  * not-%NULL and syntax highlighting is enabled (see gtk_source_buffer_set_highlight_syntax()),
  * the syntax patterns defined in @language will be used to highlight the text
  * contained in the buffer. If @language is %NULL, the text contained in the
@@ -1348,7 +1515,7 @@ gtk_source_buffer_set_language (GtkSourceBuffer   *buffer,
  * see gtk_source_buffer_set_language().  The returned object should not be
  * unreferenced by the user.
  *
- * Return value: #GtkSourceLanguage associated with the buffer, or %NULL.
+ * Return value: (transfer none): the #GtkSourceLanguage associated with the buffer, or %NULL.
  **/
 GtkSourceLanguage *
 gtk_source_buffer_get_language (GtkSourceBuffer *buffer)
@@ -1412,7 +1579,7 @@ gtk_source_buffer_ensure_highlight (GtkSourceBuffer   *buffer,
 /**
  * gtk_source_buffer_set_style_scheme:
  * @buffer: a #GtkSourceBuffer.
- * @scheme: style scheme.
+ * @scheme: (allow-none): a #GtkSourceStyleScheme or %NULL.
  *
  * Sets style scheme used by the buffer. If @scheme is %NULL no
  * style scheme is used.
@@ -1444,15 +1611,18 @@ gtk_source_buffer_set_style_scheme (GtkSourceBuffer      *buffer,
  * gtk_source_buffer_get_style_scheme:
  * @buffer: a #GtkSourceBuffer.
  *
- * Returns the #GtkSourceStyleScheme currently used in @buffer.
+ * Returns the #GtkSourceStyleScheme associated with the buffer,
+ * see gtk_source_buffer_set_style_scheme().
+ * The returned object should not be unreferenced by the user.
  *
- * Returns: the #GtkSourceStyleScheme set by
- * gtk_source_buffer_set_style_scheme(), or %NULL.
+ * Return value: (transfer none): the #GtkSourceStyleScheme associated
+ * with the buffer, or %NULL.
  **/
 GtkSourceStyleScheme *
 gtk_source_buffer_get_style_scheme (GtkSourceBuffer *buffer)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), NULL);
+
 	return buffer->priv->style_scheme;
 }
 
@@ -1560,6 +1730,27 @@ source_mark_insert (GtkSourceBuffer *buffer, GtkSourceMark *mark)
 	g_array_insert_val (buffer->priv->source_marks, idx, mark);
 }
 
+static void
+gtk_source_buffer_real_apply_tag (GtkTextBuffer     *buffer,
+                                  GtkTextTag        *tag,
+                                  const GtkTextIter *start,
+                                  const GtkTextIter *end)
+{
+	GtkSourceBuffer *source;
+
+	source = GTK_SOURCE_BUFFER (buffer);
+
+	/* We only allow the bracket match tag to be applied when we are doing
+	   it ourselves (i.e. when allow_bracket_match is TRUE). The reason for
+	   doing so is that when you copy/paste from the same buffer, the tags
+	   get pasted too. This is ok for highlighting because the region will
+	   get rehighlighted, but not for bracket matching. */
+	if (source->priv->allow_bracket_match || tag != get_bracket_match_tag (source))
+	{
+		GTK_TEXT_BUFFER_CLASS (gtk_source_buffer_parent_class)->apply_tag (buffer, tag, start, end);
+	}
+}
+
 static void
 gtk_source_buffer_real_mark_set	(GtkTextBuffer     *buffer,
 				 const GtkTextIter *location,
@@ -1621,7 +1812,7 @@ gtk_source_buffer_real_redo (GtkSourceBuffer *buffer)
 /**
  * gtk_source_buffer_create_source_mark:
  * @buffer: a #GtkSourceBuffer.
- * @name: the name of the mark, or %NULL.
+ * @name: (allow-none): the name of the mark, or %NULL.
  * @category: a string defining the mark category.
  * @where: location to place the mark.
  *
@@ -1639,7 +1830,7 @@ gtk_source_buffer_real_redo (GtkSourceBuffer *buffer)
  * Typical uses for a source mark are bookmarks, breakpoints, current
  * executing instruction indication in a source file, etc..
  *
- * Return value: a new #GtkSourceMark, owned by the buffer.
+ * Return value: (transfer none): a new #GtkSourceMark, owned by the buffer.
  *
  * Since: 2.2
  **/
@@ -1755,13 +1946,13 @@ _gtk_source_buffer_source_mark_prev (GtkSourceBuffer *buffer,
  * gtk_source_buffer_forward_iter_to_source_mark:
  * @buffer: a #GtkSourceBuffer.
  * @iter: an iterator.
- * @category: category to search for or %NULL
+ * @category: (allow-none): category to search for, or %NULL
  *
  * Moves @iter to the position of the next #GtkSourceMark of the given
- * @category. Returns #TRUE if @iter was moved. If @category is NULL, the
+ * @category. Returns %TRUE if @iter was moved. If @category is NULL, the
  * next source mark can be of any category.
  *
- * Returns: whether iter moved.
+ * Returns: whether @iter was moved.
  *
  * Since: 2.2
  **/
@@ -1814,13 +2005,13 @@ gtk_source_buffer_forward_iter_to_source_mark (GtkSourceBuffer *buffer,
  * gtk_source_buffer_backward_iter_to_source_mark:
  * @buffer: a #GtkSourceBuffer.
  * @iter: an iterator.
- * @category: category to search for or %NULL
+ * @category: (allow-none): category to search for, or %NULL
  *
  * Moves @iter to the position of the previous #GtkSourceMark of the given
- * category. Returns #TRUE if @iter was moved. If @category is NULL, the
+ * category. Returns %TRUE if @iter was moved. If @category is NULL, the
  * previous source mark can be of any category.
  *
- * Returns: whether iter moved.
+ * Returns: whether @iter was moved.
  *
  * Since: 2.2
  **/
@@ -1872,12 +2063,12 @@ gtk_source_buffer_backward_iter_to_source_mark (GtkSourceBuffer *buffer,
  * gtk_source_buffer_get_source_marks_at_iter:
  * @buffer: a #GtkSourceBuffer.
  * @iter: an iterator.
- * @category: category to search for or %NULL
+ * @category: (allow-none): category to search for, or %NULL
  *
  * Returns the list of marks of the given category at @iter. If @category
  * is %NULL it returns all marks at @iter.
  *
- * Returns: a newly allocated #GSList.
+ * Returns: (element-type GtkSource.Mark): a newly allocated #GSList.
  *
  * Since: 2.2
  **/
@@ -1887,6 +2078,8 @@ gtk_source_buffer_get_source_marks_at_iter (GtkSourceBuffer *buffer,
 					    const gchar     *category)
 {
 	GSList *marks, *l, *res;
+	
+	g_return_val_if_fail (iter != NULL, NULL);
 
 	res = NULL;
 	marks = gtk_text_iter_get_marks (iter);
@@ -1915,12 +2108,12 @@ gtk_source_buffer_get_source_marks_at_iter (GtkSourceBuffer *buffer,
  * gtk_source_buffer_get_source_marks_at_line:
  * @buffer: a #GtkSourceBuffer.
  * @line: a line number.
- * @category: category to search for or %NULL
+ * @category: (allow-none): category to search for, or %NULL
  *
  * Returns the list of marks of the given category at @line.
  * If @category is NULL, all marks at @line are returned.
  *
- * Returns: a newly allocated #GSList.
+ * Returns: (element-type GtkSource.Mark): a newly allocated #GSList.
  *
  * Since: 2.2
  **/
@@ -1967,9 +2160,9 @@ gtk_source_buffer_get_source_marks_at_line (GtkSourceBuffer *buffer,
 /**
  * gtk_source_buffer_remove_source_marks:
  * @buffer: a #GtkSourceBuffer.
- * @start: a #GtkTextIter
- * @end: a #GtkTextIter
- * @category: category to search for or NULL
+ * @start: a #GtkTextIter.
+ * @end: a #GtkTextIter.
+ * @category: (allow-none): category to search for, or %NULL.
  *
  * Remove all marks of @category between @start and @end from the buffer.
  * If @category is NULL, all marks in the range will be removed.
@@ -2027,8 +2220,8 @@ gtk_source_buffer_remove_source_marks (GtkSourceBuffer   *buffer,
 /**
  * gtk_source_buffer_iter_has_context_class:
  * @buffer: a #GtkSourceBuffer.
- * @iter: a #GtkTextIter
- * @context_class: class to search for
+ * @iter: a #GtkTextIter.
+ * @context_class: class to search for.
  *
  * Check if the class @context_klass is set on @iter.
  *
@@ -2066,12 +2259,12 @@ gtk_source_buffer_iter_has_context_class (GtkSourceBuffer   *buffer,
 /**
  * gtk_source_buffer_get_context_classes_at_iter:
  * @buffer: a #GtkSourceBuffer.
- * @iter: a #GtkTextIter
+ * @iter: a #GtkTextIter.
  *
  * Get all defined context classes at @iter.
  *
- * Returns: a new %NULL terminated array of context class names. Use
- *          #g_strfreev to free the array if it is no longer needed.
+ * Returns: (array zero-terminated=1): a new %NULL terminated array of context
+ * class names. Use #g_strfreev to free the array if it is no longer needed.
  *
  * Since: 2.10
  **/
@@ -2107,8 +2300,8 @@ gtk_source_buffer_get_context_classes_at_iter (GtkSourceBuffer   *buffer,
 /**
  * gtk_source_buffer_iter_forward_to_context_class_toggle:
  * @buffer: a #GtkSourceBuffer.
- * @iter: a #GtkTextIter
- * @context_class: the context class
+ * @iter: a #GtkTextIter.
+ * @context_class: the context class.
  *
  * Moves forward to the next toggle (on or off) of the context class. If no
  * matching context class toggles are found, returns %FALSE, otherwise %TRUE.
@@ -2152,8 +2345,8 @@ gtk_source_buffer_iter_forward_to_context_class_toggle (GtkSourceBuffer *buffer,
 /**
  * gtk_source_buffer_iter_backward_to_context_class_toggle:
  * @buffer: a #GtkSourceBuffer.
- * @iter: a #GtkTextIter
- * @context_class: the context class
+ * @iter: a #GtkTextIter.
+ * @context_class: the context class.
  *
  * Moves backward to the next toggle (on or off) of the context class. If no
  * matching context class toggles are found, returns %FALSE, otherwise %TRUE.
@@ -2194,3 +2387,53 @@ gtk_source_buffer_iter_backward_to_context_class_toggle (GtkSourceBuffer *buffer
 	}
 }
 
+/**
+ * gtk_source_buffer_set_undo_manager:
+ * @buffer: a #GtkSourceBuffer.
+ * @manager: (allow-none): A #GtkSourceUndoManager or %NULL.
+ *
+ * Set the buffer undo manager. If @manager is %NULL the default undo manager
+ * will be set.
+ **/
+void
+gtk_source_buffer_set_undo_manager (GtkSourceBuffer      *buffer,
+                                    GtkSourceUndoManager *manager)
+{
+	g_return_if_fail (GTK_IS_SOURCE_BUFFER (buffer));
+	g_return_if_fail (manager == NULL || GTK_IS_SOURCE_UNDO_MANAGER (manager));
+
+	if (manager == NULL)
+	{
+		manager = g_object_new (GTK_TYPE_SOURCE_UNDO_MANAGER_DEFAULT,
+		                        "buffer", buffer,
+		                        "max-undo-levels", buffer->priv->max_undo_levels,
+		                        NULL);
+	}
+	else
+	{
+		g_object_ref (manager);
+	}
+
+	set_undo_manager (buffer, manager);
+	g_object_unref (manager);
+
+	g_object_notify (G_OBJECT (buffer), "undo-manager");
+}
+
+/**
+ * gtk_source_buffer_get_undo_manager:
+ * @buffer: a #GtkSourceBuffer.
+ *
+ * Returns the #GtkSourceUndoManager associated with the buffer,
+ * see gtk_source_buffer_set_undo_manager().  The returned object should not be
+ * unreferenced by the user.
+ *
+ * Returns: (transfer none): the #GtkSourceUndoManager associated with the buffer, or %NULL.
+ **/
+GtkSourceUndoManager *
+gtk_source_buffer_get_undo_manager (GtkSourceBuffer *buffer)
+{
+	g_return_val_if_fail (GTK_IS_SOURCE_BUFFER (buffer), NULL);
+
+	return buffer->priv->undo_manager;
+}
