@@ -23,6 +23,7 @@
 #include "gtksourceview.h"
 #include "gtksourcelanguage-private.h"
 #include "gtksourcestyle-private.h"
+#include "gseal-gtk-compat.h"
 #include <libxml/parser.h>
 #include <string.h>
 
@@ -288,7 +289,7 @@ gtk_source_style_scheme_get_name (GtkSourceStyleScheme *scheme)
  * gtk_source_style_scheme_get_description:
  * @scheme: a #GtkSourceStyleScheme.
  *
- * Returns: @scheme description (if defined) or NULL.
+ * Returns: @scheme description (if defined), or %NULL.
  *
  * Since: 2.0
  */
@@ -304,9 +305,8 @@ gtk_source_style_scheme_get_description (GtkSourceStyleScheme *scheme)
  * gtk_source_style_scheme_get_authors:
  * @scheme: a #GtkSourceStyleScheme.
  *
- * Returns: a %NULL-terminated array containing the @scheme authors or
- * %NULL if no author is specified by the style
- * scheme.
+ * Returns: (array zero-terminated): a %NULL-terminated array containing
+ * the @scheme authors or %NULL if no author is specified by the style scheme.
  *
  * Since: 2.0
  */
@@ -326,7 +326,7 @@ gtk_source_style_scheme_get_authors (GtkSourceStyleScheme *scheme)
  * @scheme: a #GtkSourceStyleScheme.
  *
  * Returns: @scheme file name if the scheme was created parsing a
- * style scheme file or NULL in the other cases.
+ * style scheme file or %NULL in the other cases.
  *
  * Since: 2.0
  */
@@ -719,7 +719,7 @@ update_cursor_colors (GtkWidget      *widget,
 
 	if (primary != NULL && secondary == NULL)
 	{
-		secondary_color = widget->style->base[GTK_STATE_NORMAL];
+		secondary_color = gtk_widget_get_style (widget)->base[GTK_STATE_NORMAL];
 		secondary_color.red = ((gint) secondary_color.red + primary->red) / 2;
 		secondary_color.green = ((gint) secondary_color.green + primary->green) / 2;
 		secondary_color.blue = ((gint) secondary_color.blue + primary->blue) / 2;
@@ -734,7 +734,7 @@ update_cursor_colors (GtkWidget      *widget,
 
 /**
  * _gtk_source_style_scheme_apply:
- * @scheme: a #GtkSourceStyleScheme or NULL.
+ * @scheme: (allow-none): a #GtkSourceStyleScheme or %NULL.
  * @widget: a #GtkWidget to apply styles to.
  *
  * Sets text colors from @scheme in the @widget.
@@ -745,7 +745,7 @@ void
 _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 				GtkWidget            *widget)
 {
-	g_return_if_fail (!scheme || GTK_IS_SOURCE_STYLE_SCHEME (scheme));
+	g_return_if_fail (scheme == NULL || GTK_IS_SOURCE_STYLE_SCHEME (scheme));
 	g_return_if_fail (GTK_IS_WIDGET (widget));
 
 	if (scheme != NULL)
@@ -1170,6 +1170,7 @@ const gchar *
 _gtk_source_style_scheme_get_parent_id (GtkSourceStyleScheme *scheme)
 {
 	g_return_val_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme), NULL);
+
 	return scheme->priv->parent_id;
 }
 
@@ -1189,7 +1190,7 @@ _gtk_source_style_scheme_set_parent (GtkSourceStyleScheme *scheme,
 				     GtkSourceStyleScheme *parent_scheme)
 {
 	g_return_if_fail (GTK_IS_SOURCE_STYLE_SCHEME (scheme));
-	g_return_if_fail (!parent_scheme || GTK_IS_SOURCE_STYLE_SCHEME (parent_scheme));
+	g_return_if_fail (parent_scheme == NULL || GTK_IS_SOURCE_STYLE_SCHEME (parent_scheme));
 
 	if (scheme->priv->parent != NULL)
 		g_object_unref (scheme->priv->parent);
